@@ -1,5 +1,5 @@
 package Storage::Abstract::Driver;
-$Storage::Abstract::Driver::VERSION = '0.006';
+$Storage::Abstract::Driver::VERSION = '0.007';
 use v5.14;
 use warnings;
 
@@ -9,6 +9,7 @@ use Types::Common -types;
 use namespace::autoclean;
 
 use Scalar::Util qw(blessed);
+use Storage::Abstract::Handle;
 use Storage::Abstract::X;
 
 # Not using File::Spec here, because paths must be unix-like regardless of
@@ -17,6 +18,13 @@ use constant UPDIR_STR => '..';
 use constant CURDIR_STR => '.';
 use constant DIRSEP_STR => '/';
 
+has param 'readonly' => (
+	isa => Bool,
+	writer => 1,
+	lazy => 1,
+	clearer => -hidden,
+);
+
 # HELPERS
 
 # this is intentionally not portable - only drivers working on an actual
@@ -56,79 +64,21 @@ sub resolve_path
 	return join DIRSEP_STR, @path;
 }
 
-sub open_handle
-{
-	my ($self, $arg) = @_;
-	return $arg
-		if blessed $arg && $arg->isa('IO::Handle');
-
-	open my $fh, '<:raw', $arg
-		or Storage::Abstract::X::HandleError->raise((ref $arg ? '' : "$arg: ") . $!);
-
-	return $fh;
-}
-
-sub copy_handle
-{
-	my ($self, $handle_from, $handle_to) = @_;
-
-	# no extra behavior of print
-	local $\;
-
-	my $read = sub { read $_[0], $_[1], 8 * 1024 };
-	my $write = sub { print {$_[0]} $_[1] };
-
-	# can use sysread / syswrite?
-	if (fileno $handle_from != -1 && fileno $handle_to != -1) {
-		$read = sub { sysread $_[0], $_[1], 128 * 1024 };
-		$write = sub {
-			my $written = 0;
-			while ($written < $_[2]) {
-				my $res = syswrite $_[0], $_[1], $_[2], $written;
-				return undef unless defined $res;
-				$written += $res;
-			}
-
-			return 1;
-		};
-	}
-
-	my $buffer;
-	while ('copying') {
-		my $bytes = $read->($handle_from, $buffer);
-
-		Storage::Abstract::X::HandleError->raise("error reading from handle: $!")
-			unless defined $bytes;
-		last if $bytes == 0;
-		$write->($handle_to, $buffer, $bytes)
-			or Storage::Abstract::X::StorageError->raise("error during file copying: $!");
-	}
-}
-
 sub common_properties
 {
 	my ($self, $handle) = @_;
-	my $size = do {
-		if (fileno $handle == -1) {
-			my $success = (my $pos = tell $handle) >= 0;
-			$success &&= seek $handle, 0, 2;
-			$success &&= (my $res = tell $handle) >= 0;
-			$success &&= seek $handle, $pos, 0;
-
-			$success or Storage::Abstract::X::HandleError->raise($!);
-			$res;
-		}
-		else {
-			-s $handle;
-		}
-	};
 
 	return {
-		size => $size,
+		size => tied(*$handle)->size,
 		mtime => time,
 	};
 }
 
+sub refresh
+{
+	# Nothing to clear here
+}
+
 # TO BE IMPLEMENTED IN SUBCLASSES
 
 sub store_impl
@@ -171,13 +121,7 @@ sub list_impl
 sub store
 {
 	my ($self, $name, $handle) = @_;
-
-	if (ref $handle ne 'GLOB') {
-		Storage::Abstract::X::HandleError->raise('handle argument is not defined')
-			unless defined $handle;
-
-		$handle = $self->open_handle($handle);
-	}
+	local $Storage::Abstract::X::path_context = $name;
 
 	Storage::Abstract::X::Readonly->raise('storage is readonly')
 		if $self->readonly;
@@ -189,6 +133,7 @@ sub store
 sub is_stored
 {
 	my ($self, $name) = @_;
+	local $Storage::Abstract::X::path_context = $name;
 
 	return $self->is_stored_impl($self->resolve_path($name));
 }
@@ -196,27 +141,20 @@ sub is_stored
 sub retrieve
 {
 	my ($self, $name, $properties) = @_;
-	my $path = $self->resolve_path($name);
+	local $Storage::Abstract::X::path_context = $name;
 
-	Storage::Abstract::X::NotFound->raise("file $name was not found")
-		unless $self->is_stored_impl($path);
-
-	return $self->retrieve_impl($path, $properties);
+	return $self->retrieve_impl($self->resolve_path($name), $properties);
 }
 
 sub dispose
 {
 	my ($self, $name) = @_;
+	local $Storage::Abstract::X::path_context = $name;
 
 	Storage::Abstract::X::Readonly->raise('storage is readonly')
 		if $self->readonly;
 
-	my $path = $self->resolve_path($name);
-
-	Storage::Abstract::X::NotFound->raise("file $name was not found")
-		unless $self->is_stored_impl($path);
-
-	$self->dispose_impl($path);
+	$self->dispose_impl($self->resolve_path($name));
 	return;
 }
 
@@ -243,14 +181,15 @@ Storage::Abstract::Driver - Base class for drivers
 	extends 'Storage::Abstract::Driver';
 
 	# consume one of those roles
-	with 'Storage::Abstract::Role::Driver';
-	with 'Storage::Abstract::Role::Metadriver';
+	with 'Storage::Abstract::Role::Driver::Basic';
+	with 'Storage::Abstract::Role::Driver::Meta';
 
 	# these methods need implementing
 	sub store_impl { ... }
 	sub is_stored_impl { ... }
 	sub retrieve_impl { ... }
 	sub dispose_impl { ... }
+	sub list_impl { ... }
 
 =head1 DESCRIPTION
 
@@ -260,8 +199,8 @@ methods which must be implemented in the subclasses, and a couple of helpers
 which may be used or reimplemented in the subclasses when needed.
 
 This class should never be instantiated directly. Its subclasses should consume
-one of the roles, either L<Storage::Abstract::Role::Driver> or
-L<Storage::Abstract::Role::Metadriver>.
+one of the roles, either L<Storage::Abstract::Role::Driver::Basic> or
+L<Storage::Abstract::Role::Driver::Meta>.
 
 =head1 INTERFACE
 
@@ -274,12 +213,13 @@ These attributes are common to all drivers.
 Boolean - whether this driver is readonly. False by default. May be changed
 using C<set_readonly>.
 
-This attribute is not applicable to metadrivers. This type of drivers don't
-store its own readonly status, but instead reports the status of its source via
-C<readonly>. Calling C<set_readonly> on metadrivers will call C<set_readonly>
-of the underlying driver. If the metadriver holds more than one source (for
-example L<Storage::Abstract::Driver::Composite>), calling C<set_readonly> will
-throw an exception.
+This attribute is not applicable to meta drivers. This type of drivers don't
+store its own readonly status, so this attribute is used as a cache for the
+underlying drivers C<readonly> status. Calling C<set_readonly> on meta drivers
+will call C<set_readonly> of the underlying driver and refresh the cache. If
+the meta driver holds more than one source (for example
+L<Storage::Abstract::Driver::Composite>), calling C<set_readonly> will throw an
+exception.
 
 =head2 Helper methods
 
@@ -295,22 +235,6 @@ is called, before the path is used for anything. As such, it can be
 reimplemented in a driver class to modify its behavior (see
 L<Storage::Abstract::Driver::Directory> for an example).
 
-=head3 open_handle
-
-	$fh = $obj->open_handle(\$content);
-	$fh = $obj->open_handle($filename);
-
-This tries to create a readonly, binary handle from its argument. It will not
-do anything if the argument is a class of L<IO::Handle>.
-
-=head3 copy_handle
-
-	$obj->copy_handle($fh_from, $fh_to)
-
-This copies the data from C<$fh_from> to C<$fh_to>. Based on the C<fileno>
-result on the handles, it uses either C<sysread> + C<syswrite> or C<read> +
-C<print>. Use this to move data between filehandles in driver classes.
-
 =head3 common_properties
 
 	my $properties = $obj->common_properties($handle);
@@ -331,7 +255,11 @@ These methods must be reimplemented in driver classes:
 	store_impl($path, $fh)
 
 The implementation of storing a new file in the storage. It will be passed a
-normalized path and an opened file handle. Its return value will be ignored.
+normalized path and an open file handle. For drivers implementing
+L<Storage::Abstract::Role::Driver::Basic>, C<$fh> will be a tied object of
+L<Storage::Abstract::Handle>.
+
+Its return value will be ignored.
 
 =item * C<is_stored_impl>
 
@@ -351,8 +279,14 @@ fetch properties when the second argument is undefined (if such optimization is
 possible). Every driver should include at least the same keys as returned by
 L</common_properties>.
 
-It should not check C<is_stored> - it will never be called without checking
-C<is_stored> first. Must return an opened file handle to the file.
+Basic drivers should not check C<is_stored> - it will never be called without checking
+C<is_stored> first. Must return an open file handle to the file. The file
+handle should be rewound to the beginning (ready to be read without calling
+C<seek>) and it should read data into memory lazily regardless of the
+underlying storage type. It is recommended that the file handle is a tied
+object of L<Storage::Abstract::Handle> or its subclass. Calling
+C<retrieve_impl> by itself should not cause the storage to perform any IO
+operations, so that it can be used just to fetch C<%properties> efficiently.
 
 =item * C<dispose_impl>
 
@@ -360,7 +294,7 @@ C<is_stored> first. Must return an opened file handle to the file.
 
 The implementation of disposing a file. First argument is a normalized path.
 
-It should not check C<is_stored> - it will never be called without checking
+Basic drivers should not check C<is_stored> - it will never be called without checking
 C<is_stored> first. Its return value will be ignored.
 
 =item * C<list_impl>
